<?php
/**
 *
 *  AbstractApi.php
 *
 * @author   dlo
 * @copyright  CYBERDAY GmbH
 * @license    CYBERDAY GmbH
 * @version    0.1
 * @since    18.03.2016
 *
 *                          _(__)_        V
 *                        '-e e -'__,--.__)
 *                         (o_o)        )
 *                            \. /___.  |
 *                            ||| _)/_)/
 *                         dlo//_(/_(/_(
 *
 **/
namespace Cyberday\ApiItRechtKanzlei\Controller;

use Cyberday\ApiItRechtKanzlei\Helper\Action\Accountlist as HelperAccountlist;
use Cyberday\ApiItRechtKanzlei\Helper\Action\Push as HelperPush;
use Cyberday\ApiItRechtKanzlei\Helper\Data;
use Cyberday\ApiItRechtKanzlei\Model\Api\Exception as ApiException;
use Cyberday\ApiItRechtKanzlei\Model\Api\Log as ApiLog;
use Magento\Framework\App\Action\Action;
use Magento\Framework\App\Action\Context;
use Magento\Framework\App\RequestInterface;
use Magento\Framework\Controller\Result\Raw as ResultRaw;
use Psr\Log\LoggerInterface;
use \Throwable;
use \Exception;

/**
 * Class AbstractApi
 *
 * @package Cyberday\ApiItRechtKanzlei\Controller
 *          @SuppressWarnings(PHPMD.NumberOfChildren)
 */
abstract class AbstractApi extends Action implements ApiInterface
{

    const STATUS_ERROR = 'error';

    const STATUS_SUCCESS = 'success';

    const ACTION_PUSH = 'push';

    const ACTION_ACCOUNTLIST = 'getaccountlist';

    /**
     *
     * @var \Cyberday\ApiItRechtKanzlei\Helper\Data
     */
    protected $_helper;

    /**
     *
     * @var \Cyberday\ApiItRechtKanzlei\Helper\Action\Push
     */
    protected $_helperPush;

    /**
     *
     * @var \Cyberday\ApiItRechtKanzlei\Helper\Action\Accountlist
     */
    protected $_helperAccountlist;

    /**
     *
     * @var \Psr\Log\LoggerInterface
     */
    protected $_logger;

    /**
     *
     * @var Magento\Framework\Controller\Result\Raw
     */
    protected $_resultRawFactory;

    /**
     *
     * @var Cyberday\ApiItRechtKanzlei\Model\Api\Log
     */
    protected $_apiLog;

    /**
     *
     * @param Context $context
     * @param Data $helper
     * @param ResultRaw $resultRawFactory
     * @param HelperPush $helperPush
     * @param HelperAccountlist $helperAccountlist
     * @param LoggerInterface $logger
     * @param ApiLog $apiLog
     */
    public function __construct(Context $context, Data $helper, ResultRaw $resultRawFactory, HelperPush $helperPush, HelperAccountlist $helperAccountlist, LoggerInterface $logger, ApiLog $apiLog)
    {
        parent::__construct($context);
        $this->_helper = $helper;
        $this->_helperPush = $helperPush;
        $this->_helperAccountlist = $helperAccountlist;
        $this->_resultRawFactory = $resultRawFactory;
        $this->_logger = $logger;
        $this->_apiLog = $apiLog;
                 
        // CsrfAwareAction Magento2.3 compatibility
        if (interface_exists('\Magento\Framework\App\CsrfAwareActionInterface')) {
            $request = $this->getRequest();
            if ($request instanceof RequestInterface && $request->isPost() && empty($request->getParam('form_key'))) {
                $formKey = $this->_objectManager->get(\Magento\Framework\Data\Form\FormKey::class);
                $request->setParam('form_key', $formKey->getFormKey());
            }
        }
        //
    }

    /**
     * Dispatch request
     *
     * @param RequestInterface $request
     * @return ResponseInterface
     * @throws NotFoundException
     */
    public function dispatch(RequestInterface $request)
    {
        $helper = $this->_helper;

        $startTime = microtime(true);

        try {
            $this->preDispatchChecks($request);
            $returnResult = parent::dispatch($request);
        } catch (\Exception $exception) {

            $dataObj = $helper->getDefaultResponseObject();
            $dataObj->setStatus(self::STATUS_ERROR);
            $dataObj->setError((string) $exception->getCode());
            if (! ($exception instanceof ApiException) || ! $exception->getCode()) {
                if(! $exception->getCode()) {
                    $dataObj->setException($exception->getMessage());
                }
                $dataObj->setError((string) ApiException::ERROR_OTHER);
                $this->_logger->critical($exception);
            }

            $returnString = $this->convertResult($dataObj, $this->_objectManager->create('Cyberday\ApiItRechtKanzlei\Model\Api\Request\DataObject'));
            $this->_apiLog->setResponse($returnString);

            $result = $this->_resultRawFactory;
            $returnResult = $result->setContents($returnString);
        }

        $runTime = round(microtime(true) - $startTime, 4);

        try {
            $remoteAddressHelper = $this->_objectManager->create('Magento\Framework\HTTP\PhpEnvironment\RemoteAddress', [
                'alternativeHeaders' => [
                    'HTTP_X_FORWARDED_FOR'
                ]
            ]);

            $apiLog = $this->_apiLog;
            $apiLog->setRequest($this->getRequestContent($request));
            $apiLog->setRuntime($runTime);
            $apiLog->setIp($remoteAddressHelper->getRemoteAddress());

            if ($helper->isApiLoggingEnabled()) {
                $apiLog->save();
            }
        } catch (Exception $exception) {
            $this->_logger->critical($exception);
        } catch (Throwable $exception) {
            $this->_logger->critical($exception);
        }

        return $returnResult;
    }

    /**
     * @return \Magento\Framework\App\ResponseInterface|ResultRaw|\Magento\Framework\Controller\ResultInterface
     * @throws ApiException
     */
    public function execute()
    {
        $convertedRequest = $this->getConvertedRequest($this->getRequest());
        $helper = $this->_helper;

        if (! $helper->validateUser($convertedRequest->getUserUsername(), $convertedRequest->getUserPassword())) {
            throw new ApiException('Fehler beim Authentifizieren an Ihrer Schnittstelle durch user_username/user_password', ApiException::ERROR_TOKEN);
        }

        $action = $convertedRequest->getAction();

        $dataObj = $helper->getDefaultResponseObject();
        $dataObj->setStatus(self::STATUS_SUCCESS);

        switch ($action) {
            case self::ACTION_PUSH:
                if (! $helper->validateMultiShop($convertedRequest->getUserAccountId())) {
                    throw new ApiException('Wert für user_account_id wird benötigt (Multishop-System), ist aber leer oder nicht gültig.', ApiException::ERROR_USER_ACCOUNT_ID);
                }

                $this->_helperPush->proccess($convertedRequest, $dataObj);
                break;

            case self::ACTION_ACCOUNTLIST:
                $this->_helperAccountlist->proccess($convertedRequest, $dataObj);
                break;

            default:
                throw new ApiException('Wert für action ist leer oder das Kommando wird (noch) nicht unterstützt.', ApiException::ERROR_ACTION);
        }

        $returnString = $this->convertResult($dataObj, $convertedRequest);
        $this->_apiLog->setResponse($returnString);

        $result = $this->_resultRawFactory;
        return $result->setContents($returnString);
    }
}
