<?php
/**
 *
 *  Interface.php
 *
 * @author   dlo
 * @copyright  CYBERDAY GmbH
 * @license    CYBERDAY GmbH
 * @version    0.1
 * @since    18.03.2016
 *
 *                          _(__)_        V
 *                        '-e e -'__,--.__)
 *                         (o_o)        )
 *                            \. /___.  |
 *                            ||| _)/_)/
 *                         dlo//_(/_(/_(
 *
 **/
namespace Cyberday\ApiItRechtKanzlei\Controller\Api;

use Cyberday\ApiItRechtKanzlei\Controller\AbstractApi;
use Cyberday\ApiItRechtKanzlei\Model\Api\Exception as ApiException;
use Cyberday\ApiItRechtKanzlei\Model\Api\Request\DataObject as RequestDataObject;

use Magento\Framework\App\RequestInterface;
use Magento\Framework\DataObject;

class Xml extends AbstractApi
{

    const REQUEST_CONTENT_PARAM = 'xml';

    /**
     *
     * {@inheritDoc}
     *
     * @see \Cyberday\ApiItRechtKanzlei\Controller\ApiInterface::preDispatchChecks()
     */
    public function preDispatchChecks(RequestInterface $request)
    {
        if (!$request->isPost()) {
            throw new ApiException('no post request', ApiException::ERROR_XML);
        }
        
        $xml = $request->getParam(self::REQUEST_CONTENT_PARAM, false);
        if (empty($xml)) {
            throw new ApiException('no "' . self::REQUEST_CONTENT_PARAM . '" param', ApiException::ERROR_XML);
        }
    }

    /**
     *
     * {@inheritDoc}
     *
     * @see \Cyberday\ApiItRechtKanzlei\Controller\ApiInterface::getRequestContent()
     */
    public function getRequestContent(RequestInterface $request)
    {
        return $request->getParam(self::REQUEST_CONTENT_PARAM);
    }

    /**
     *
     * {@inheritDoc}
     *
     * @see \Cyberday\ApiItRechtKanzlei\Controller\ApiInterface::getConvertedRequest()
     */
    public function getConvertedRequest(RequestInterface $request)
    {
        $content = $this->getRequestContent($request);
        
        try {
            $xmlParser = $this->_objectManager->create('Magento\Framework\Xml\Parser');
            $xmlParser->loadXML($content);
            $convertedRequest = $xmlParser->xmlToArray();
        } catch (\Exception $e) {
            throw new ApiException('xml error: ' . $e->getMessage() . ' - ' . $e->getTraceAsString(), ApiException::ERROR_XML);
        }
        
        if (! isset($convertedRequest['api'])) {
            throw new ApiException('no "api" offset in data', ApiException::ERROR_XML);
        }
        
        $convertedObj = $this->_objectManager->create('Cyberday\ApiItRechtKanzlei\Model\Api\Request\DataObject');
        $convertedObj->setData($convertedRequest['api']);
        
        return $convertedObj;
    }

    /**
     *
     * {@inheritDoc}
     *
     * @see \Cyberday\ApiItRechtKanzlei\Controller\ApiInterface::convertResult()
     */
    public function convertResult(DataObject $data, RequestDataObject $convertedRequest)
    {
        /* @var \Magento\Framework\Xml\Generator $xmlGenerator */
        $xmlGenerator = $this->_objectManager->create('Magento\Framework\Xml\Generator');
        $array = $data->toArray();
        
        $result = [
            'response' => $array
        ];

        if ($convertedRequest->getAction() == AbstractApi::ACTION_ACCOUNTLIST) {
            $result = $array;
            $xmlGenerator->setIndexedArrayItemName('account');
        }

        $response = $xmlGenerator->arrayToXml($result);

        if($this->_helper->versionCompare('2.3.3','>=')) {
            return $response->__toString();
        }

        return $response;
    }
}
