<?php
/**
 *
 *  Blocks.php
 *
 * @author   dlo
 * @copyright  CYBERDAY GmbH
 * @license    CYBERDAY GmbH
 * @version    0.1
 * @since    21.03.2016
 *
 *                          _(__)_        V
 *                        '-e e -'__,--.__)
 *                         (o_o)        )
 *                            \. /___.  |
 *                            ||| _)/_)/
 *                         dlo//_(/_(/_(
 *
 **/
namespace Cyberday\ApiItRechtKanzlei\Helper\Action;

use Cyberday\ApiItRechtKanzlei\Helper\AbstractAction;
use Cyberday\ApiItRechtKanzlei\Model\Api\Request\DataObject as RequestDataObject;
use Cyberday\ApiItRechtKanzlei\Model\Api\Exception as ApiException;
use Cyberday\ApiItRechtKanzlei\Helper\Data;
use Magento\Framework\ObjectManagerInterface;
use Magento\Framework\App\Helper\Context;
use Magento\Framework\DataObject;
use Magento\Framework\App\Cache\Type\Config as ConfigCache;

class Push extends AbstractAction
{

    const INCLUDE_STATIC_BLOCK = '{{block class="Magento\\Cms\\Block\\Block" block_id="%s"}}';

    const TYPE_AGB = 'agb';

    const TYPE_DATENSCHUTZ = 'datenschutz';

    const TYPE_WIDERRUF = 'widerruf';

    const TYPE_IMPRESSUM = 'impressum';

    const IDENT_PREFIX = 'itrk-';

    /**
     *
     * @var \Magento\Framework\ObjectManagerInterface
     */
    protected $_objectManager;

    /**
     *
     * @var \Cyberday\ApiItRechtKanzlei\Helper\Data
     */
    protected $_helper;

    /**
     *
     * @var \Magento\Framework\App\Cache\Type\Config
     */
    protected $_configCache;

    protected $_types = [
        self::TYPE_AGB,
        self::TYPE_DATENSCHUTZ,
        self::TYPE_WIDERRUF,
        self::TYPE_IMPRESSUM
    ];

    protected $_typesNoPdf = [
        self::TYPE_IMPRESSUM
    ];

    /**
     *
     * @param Context $context
     * @param ObjectManagerInterface $objectManager
     * @param Data $helper
     * @param ConfigCache $configCache
     *
     */
    public function __construct(Context $context, ObjectManagerInterface $objectManager, Data $helper, ConfigCache $configCache)
    {
        parent::__construct($context);
        $this->_objectManager = $objectManager;
        $this->_helper = $helper;
        $this->_configCache = $configCache;
    }

    protected function _refreshCache()
    {
        try {
            $configCache = $this->_configCache;
            $configCache->clean();
        } catch (\Exception $e) {}
    }

    /**
     *
     * @param string $type
     * @return boolean
     */
    protected function _validateType($type)
    {
        if (! in_array($type, $this->_types)) {
            return false;
        }
        return true;
    }

    /**
     *
     * @param string $type
     * @return boolean
     */
    protected function _validateHtml($html)
    {
        if (empty($html) || strlen($html) < 50) {
            return false;
        }

        return true;
    }

    /**
     *
     * @param string $type
     * @return boolean
     */
    protected function _validatePdf($pdf)
    {
        if (empty($pdf)) {
            return false;
        }

        return true;
    }

    protected function _needPdf($type)
    {
        if (in_array($type, $this->_typesNoPdf)) {
            return false;
        }

        return true;
    }

    /**
     *
     * @param RequestDataObject $requestDataObject
     * @throws ApiException
     */
    protected function _validate(RequestDataObject $requestDataObject)
    {
        if (! $this->_validateType($requestDataObject->getRechtstextType())) {
            throw new ApiException('Wert für rechtstext_type ist leer oder gesendeter Typ wird (noch) nicht untersützt', ApiException::ERROR_LAW_TYPE);
        }

        if (! $this->_validateHtml($requestDataObject->getRechtstextHtml())) {
            throw new ApiException('Wert für rechtstext_html ist leer oder < 50 Zeichen', ApiException::ERROR_LAW_HTML);
        }

        if (! $this->_validateHtml($requestDataObject->getRechtstextText())) {
            throw new ApiException('Wert für rechtstext_text ist leer oder < 50 Zeichen', ApiException::ERROR_LAW_TEXT);
        }

        if ($this->_needPdf($requestDataObject->getRechtstextType())) {
            if (! $this->_validatePdf($requestDataObject->getRechtstextPdfUrl())) {
                throw new ApiException('PDF-Datei Wert ist leer, obwohl eine PDF-Datei benötigt wird', ApiException::ERROR_LAW_PDF);
            }
        }
    }

    /**
     *
     * @param string $type
     * @param string $pdfUrl
     * @param string $pdfHash
     * @param int $store
     * @throws ApiException
     */
    protected function _proccessPdf($type, $pdfUrl, $pdfHash, $store)
    {
        $helper = $this->_helper;

        try {
            /*@var $curl \Magento\Framework\HTTP\Client\Curl */
            $curl = $this->_objectManager->create('Magento\Framework\HTTP\Client\Curl');
            $curl->get($pdfUrl);
            $pdfContent = $curl->getBody();
        } catch (\Exception $e) {
            throw new ApiException($e->getMessage(), ApiException::ERROR_LAW_PDF);
        }

        if (empty($pdfContent)) {
            throw new ApiException('PDF-Datei ist leer.', ApiException::ERROR_LAW_PDF);
        }

        $pdfDir = $helper->getPdfDir($store);
        $filename = $type . '.pdf';
        $filenameTemp = $type . '_temp.pdf';

        $completeFilename = $pdfDir . $filename;
        $completeFilenameTemp = $pdfDir . $filenameTemp;

        if (@file_put_contents($completeFilenameTemp, $pdfContent) === false) {
            throw new ApiException('PDF-Datei konnte nicht gespeichert werden.', ApiException::ERROR_LAW_PDF);
        }

        if (function_exists('mime_content_type')) {
            $fileType = mime_content_type($completeFilenameTemp);
            if ($fileType != 'application/pdf') {
                @unlink($completeFilenameTemp);
                throw new ApiException('PDF-Datei ist keine PDF-Datei.', ApiException::ERROR_LAW_PDF);
            }
        }

        $md5 = md5($pdfContent);
        if ($md5 != $pdfHash) {
            @unlink($completeFilenameTemp);
            throw new ApiException('Der MD5-Hash der ggf. unter rechtstext_pdf_url abgerufenen Datei ist nicht korrekt', ApiException::ERROR_LAW_PDF_HASH);
        }

        if (! @copy($completeFilenameTemp, $completeFilename)) {
            @unlink($completeFilenameTemp);
            throw new ApiException('PDF-Datei konnte nicht gespeichert werden.', ApiException::ERROR_LAW_PDF);
        }

        @unlink($completeFilenameTemp);
        $helper->setPdfByType($type, $store, $filename);
    }

    /**
     *
     * @param string $code
     * @param int $errorCode
     * @throws ApiException
     */
    protected function _validateCountryCode($code, $errorCode)
    {
        $code = trim($code);
        if (empty($code)) {
            throw new ApiException('Country/Language nicht korrekt.', $errorCode);
        }

        if (! preg_match('/^[a-zA-Z]{2}$/', $code)) {
            throw new ApiException('Country/Language nicht korrekt.', $errorCode);
        }
    }

    /**
     *
     * {@inheritdoc}
     *
     * @see \Cyberday\ApiItRechtKanzlei\Helper\ActionInterface::proccess()
     */
    public function proccess(RequestDataObject $requestDataObject, DataObject $dataObject)
    {
        $helper = $this->_helper;
        $refreshCache = false;
        $type = $requestDataObject->getRechtstextType();
        $country = $requestDataObject->getRechtstextCountry();
        $language = $requestDataObject->getRechtstextLanguage();

        $this->_validate($requestDataObject);
        $this->_validateCountryCode($language, ApiException::ERROR_LANGUAGE);
        $this->_validateCountryCode($country, ApiException::ERROR_LAW_COUNTRY);

        $identifier = self::IDENT_PREFIX . $type . '-' . $country . '-' . $language;
        $identifierCmsPage = strtolower($identifier);

        $store = \Magento\Store\Model\Store::DEFAULT_STORE_ID;
        if ($requestDataObject->getUserAccountId()) {
            $store = $requestDataObject->getUserAccountId();
        }

        if ($this->_needPdf($type)) {
            $this->_proccessPdf($type, $requestDataObject->getRechtstextPdfUrl(), $requestDataObject->getRechtstextPdfMd5Hash(), $store);
        }

        $completeTitle = [];
        $completeTitle[] = $type;
        $completeTitle[] = $country;
        $completeTitle[] = $language;

        $completeTitle = implode(' - ', $completeTitle);

        $blockId = $helper->getBlockByType($type, $country, $language, $store);

        /* @var $blockResourceModel \Magento\Cms\Model\ResourceModel\Block\ResourceModel\Page */
        $blockResourceModel = $this->_objectManager->create('Magento\Cms\Model\ResourceModel\Block');
        /* @var $blockCollection \Magento\Cms\Model\ResourceModel\Block\Collection */
        $blockCollection = $this->_objectManager->create('Magento\Cms\Model\ResourceModel\Block\Collection');
        $blockCollection->addStoreFilter($store);

        if ($blockId) {
            $blockCollection->addFieldToFilter('main_table.block_id', $blockId);
        } else {
            $blockCollection->addFieldToFilter('main_table.identifier', $identifier);
        }

        /* @var $blockModel \Magento\Cms\Model\Block */
        $blockModel = $blockCollection->getFirstItem();

        if (! $blockModel->getId()) {
            $refreshCache = true;
            $blockModel->setIdentifier($identifier);
            $blockModel->setTitle($completeTitle);
            $blockModel->setIsActive(1);
        }

        $blockModel->setStores([
            $store
        ]);
        $blockModel->setUpdateTime(null);

        $blockModel->setContent($requestDataObject->getRechtstextHtml());
        $blockResourceModel->save($blockModel);

        $helper->setBlockByType($type, $country, $language, $store, $blockModel->getId());

        $pageId = $helper->getPageByType($type, $country, $language, $store);

        /* @var $resourceModel \Magento\Cms\Model\ResourceModel\Page\ResourceModel\Page */
        $pageResourceModel = $this->_objectManager->create('Magento\Cms\Model\ResourceModel\Page');
        /* @var $pageCollection \Magento\Cms\Model\ResourceModel\Page\Collection */
        $pageCollection = $this->_objectManager->create('Magento\Cms\Model\ResourceModel\Page\Collection');
        $pageCollection->addStoreFilter($store);

        if ($pageId) {
            $pageCollection->addFieldToFilter('main_table.page_id', $pageId);
        } else {
            $pageCollection->addFieldToFilter('main_table.identifier', $identifierCmsPage);
        }

        /* @var $pageModel \Magento\Cms\Model\Page */
        $pageModel = $pageCollection->getFirstItem();

        if (! $pageModel->getId()) {
            $refreshCache = true;
            $pageModel->setStoreId($store);
            $pageModel->setIdentifier($identifierCmsPage);
            $pageModel->setTitle($completeTitle);
            $pageModel->setPageLayout('1column');
            $pageModel->setIsActive(1);
        }

        $pageModel->setStores([
            $store
        ]);
        $staticBlockInclude = sprintf(self::INCLUDE_STATIC_BLOCK, $blockModel->getId());
        $pageModel->setContent($staticBlockInclude);
        $pageResourceModel->save($pageModel);

        $helper->setPageByType($type, $country, $language, $store, $pageModel->getId());

        if ($refreshCache) {
            $this->_refreshCache();
        }
    }
}
