<?php
/**
 *
 *  Data.php
 *
 * @author   dlo
 * @copyright  CYBERDAY GmbH
 * @license    CYBERDAY GmbH
 * @version    0.1
 * @since    18.03.2016
 *
 *                          _(__)_        V
 *                        '-e e -'__,--.__)
 *                         (o_o)        )
 *                            \. /___.  |
 *                            ||| _)/_)/
 *                         dlo//_(/_(/_(
 *
 **/

namespace Cyberday\ApiItRechtKanzlei\Helper;

use Magento\Framework\App\Helper\AbstractHelper;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\Config\MutableScopeConfigInterface;
use Magento\Framework\App\ProductMetadata;
use Magento\Framework\Module\ResourceInterface;
use Magento\Framework\DataObject;
use Magento\Framework\ObjectManagerInterface;
use Magento\Config\Model\ResourceModel\Config as ConfigSaveHelper;
use Magento\Store\Model\ScopeInterface;
use Cyberday\ApiItRechtKanzlei\Model\Api\Exception as ApiException;

class Data extends AbstractHelper
{

    const CONFIG_PATH = 'itrk';

    const CONFIG_GLOBAL_API = 'global_api';

    const CONFIG_LOG = 'log';

    const CONFIG_BLOCK = 'block';

    const CONFIG_PAGE = 'page';

    const CONFIG_PDF = 'pdf';

    const PDF_DIR = '/itrk_pdf/';

    /**
     *
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */
    protected $_config;

    /**
     *
     * @var \Magento\Framework\Module\ResourceInterface
     */
    protected $_resourceInterface;

    /**
     *
     * @var \Magento\Framework\ObjectManagerInterface
     */
    protected $_objectManager;

    /**
     *
     * @var \Magento\Framework\App\ProductMetadata
     */
    protected $_productMetadata;

    /**
     * @var ConfigSaveHelper
     */
    protected $_configSaveHelper;

    /**
     *
     * @param MutableScopeConfigInterface $scopeConfig
     * @param ResourceInterface $resourceInterface
     * @param ConfigSaveHelper $configSaveHelper
     * @param ObjectManagerInterface $objectManager
     * @param ProductMetadata $productMetadata
     */
    public function __construct(
        MutableScopeConfigInterface $scopeConfig,
        ResourceInterface $resourceInterface,
        ConfigSaveHelper $configSaveHelper,
        ObjectManagerInterface $objectManager,
        ProductMetadata $productMetadata
    )
    {

        $this->_config = $scopeConfig;
        $this->_configSaveHelper = $configSaveHelper;
        $this->_resourceInterface = $resourceInterface;
        $this->_objectManager = $objectManager;
        $this->_productMetadata = $productMetadata;
    }

    /**
     *
     * @param string $key
     * @return null|string
     */
    protected function _getGlobalConfigValue($key)
    {
        return $this->_config->getValue(self::CONFIG_PATH . '/' . self::CONFIG_GLOBAL_API . '/' . $key);
    }

    /**
     *
     * @param string $key
     * @return null|string
     */
    protected function _getBlockConfigValue($key, $store)
    {
        $scope = $this->_getScope($store);
        return $this->_config->getValue(self::CONFIG_PATH . '/' . self::CONFIG_BLOCK . '/' . $key, $scope, $store);
    }

    /**
     *
     * @param string $key
     * @return null|string
     */
    protected function _getPageConfigValue($key, $store)
    {
        $scope = ScopeConfigInterface::SCOPE_TYPE_DEFAULT;
        if ($store != 0) {
            $scope = ScopeInterface::SCOPE_STORES;
        }

        return $this->_config->getValue(self::CONFIG_PATH . '/' . self::CONFIG_PAGE . '/' . $key, $scope, $store);
    }

    /**
     *
     * @param string $key
     * @return null|string
     */
    protected function _getPdfConfigValue($key, $store)
    {
        $scope = $this->_getScope($store);
        return $this->_config->getValue(self::CONFIG_PATH . '/' . self::CONFIG_PDF . '/' . $key, $scope, $store);
    }

    /**
     *
     * @return null|string
     */
    protected function _getUserUsername()
    {
        return $this->_getGlobalConfigValue('username');
    }

    /**
     *
     * @return null|string
     */
    protected function _getUserPassword()
    {
        return $this->_getGlobalConfigValue('password');
    }

    /**
     *
     * @param string $dir
     * @throws ApiException
     */
    protected function _mkdir($dir)
    {
        if (!is_dir($dir)) {
            if (!mkdir($dir)) {
                throw new ApiException('Kann PDF Ordner nicht erstellen.', ApiException::ERROR_LAW_PDF);
            }
        }
    }

    /**
     *
     * @param int $store
     */
    protected function _getScope($store)
    {
        $scope = ScopeConfigInterface::SCOPE_TYPE_DEFAULT;
        if ($store != 0) {
            $scope = ScopeInterface::SCOPE_STORES;
        }
        return $scope;
    }

    /**
     *
     * @return \Magento\Store\Model\ResourceModel\Store\Collection
     */
    protected function _getStoreCollection()
    {
        /*@var $storeModel \Magento\Store\Model\Store */
        $storeModel = $this->_objectManager->create('Magento\Store\Model\Store');
        $storeCollection = $storeModel->getCollection();
        $storeCollection->setLoadDefault(false);
        return $storeCollection;
    }

    /**
     *
     * @return boolean
     */
    public function isApiLoggingEnabled()
    {
        return $this->_config->getValue(self::CONFIG_PATH . '/' . self::CONFIG_LOG . '/api_calls');
    }

    /**
     *
     * @return string
     */
    public function getShopVersion()
    {
        return $this->_productMetadata->getVersion();
    }

    /**
     *
     * @return \Magento\Framework\Module\false|string
     */
    public function getModulVersion()
    {
        return $this->_resourceInterface->getDbVersion('Cyberday_ApiItRechtKanzlei');
    }

    /**
     *
     * @param string $type
     * @param string $country
     * @param string $language
     * @param int $store
     */
    public function getBlockByType($type, $country, $language, $store)
    {
        $key = $type . '_' . $country . '_' . $language;
        return $this->_getBlockConfigValue($key, $store);
    }

    /**
     *
     * @param string $type
     * @param string $country
     * @param string $language
     * @param int $store
     * @param string $value
     */
    public function setBlockByType($type, $country, $language, $store, $value)
    {
        $key = $type . '_' . $country . '_' . $language;
        $scope = $this->_getScope($store);
        $this->_configSaveHelper->saveConfig(self::CONFIG_PATH . '/' . self::CONFIG_BLOCK . '/' . $key, $value, $scope, $store);
    }

    /**
     *
     * @param string $type
     * @param string $country
     * @param string $language
     * @param int $store
     */
    public function getPageByType($type, $country, $language, $store)
    {
        $key = $type . '_' . $country . '_' . $language;
        return $this->_getPageConfigValue($key, $store);
    }

    /**
     *
     * @param string $type
     * @param string $country
     * @param string $language
     * @param int $store
     * @param string $value
     */
    public function setPageByType($type, $country, $language, $store, $value)
    {
        $key = $type . '_' . $country . '_' . $language;
        $scope = $this->_getScope($store);
        $this->_configSaveHelper->saveConfig(self::CONFIG_PATH . '/' . self::CONFIG_PAGE . '/' . $key, $value, $scope, $store);
    }

    /**
     *
     * @param string $type
     * @param int $store
     */
    public function getPdfByType($type, $store)
    {
        return $this->_getPdfConfigValue($type, $store);
    }

    /**
     *
     * @param string $type
     * @param int $store
     * @param string $value
     */
    public function setPdfByType($type, $store, $value)
    {
        $scope = $this->_getScope($store);
        $this->_configSaveHelper->saveConfig(self::CONFIG_PATH . '/' . self::CONFIG_PDF . '/' . $type, $value, $scope, $store);
    }

    /**
     *
     * @param string $username
     * @param string $password
     * @return boolean
     */
    public function validateUser($username, $password)
    {
        if ($username != $this->_getUserUsername()) {
            return false;
        }

        if ($password != md5($this->_getUserPassword())) {
            return false;
        }

        return true;
    }

    /**
     *
     * @return boolean
     */
    public function isMultiShop()
    {
        return true;
    }

    /**
     *
     * @param string $user_account_id
     */
    public function validateMultiShop($user_account_id = '')
    {
        if (!$this->isMultiShop() && empty($user_account_id)) {
            return true;
        }

        if (!$this->isMultiShop() && !empty($user_account_id)) {
            return false;
        }

        if ($this->isMultiShop() && empty($user_account_id)) {
            return false;
        }

        $storeCollection = $this->_getStoreCollection();
        $store = $storeCollection->getItemByColumnValue('store_id', $user_account_id);
        if (!$store) {
            return false;
        }

        return true;
    }

    /**
     * @param $store
     * @return string
     * @throws ApiException
     */
    public function getPdfDir($store)
    {
        /*@var $storeModel \Magento\Store\Model\Store */
        $storeModel = $this->_objectManager->create('Magento\Store\Model\Store');
        $mediaDir = $storeModel->getBaseMediaDir();

        $pdfDir = $mediaDir . self::PDF_DIR;
        $this->_mkdir($pdfDir);

        $pdfDir .= $store . '/';
        $this->_mkdir($pdfDir);

        return $pdfDir;
    }

    /**
     *
     * @param string $type
     * @param int $store
     * @return boolean|string
     */
    public function getPdfFileByType($type, $store)
    {
        $pdf = $this->getPdfByType($type, $store);
        if (empty($pdf)) {
            return false;
        }

        $pdfDir = $this->getPdfDir($store);
        $file = $pdfDir . $pdf;

        if (!file_exists($file)) {
            return false;
        }

        return $file;
    }

    /**
     *
     * @param int $store
     * @return array
     */
    public function getPdfsOrderEmail($store)
    {
        $types = [
            \Cyberday\ApiItRechtKanzlei\Helper\Action\Push::TYPE_AGB,
            \Cyberday\ApiItRechtKanzlei\Helper\Action\Push::TYPE_WIDERRUF
        ];

        $attachments = [];

        foreach ($types as $type) {
            $pdf = $this->getPdfFileByType($type, $store);
            if (!$pdf) {
                continue;
            }
            $attachments[] = $pdf;
        }

        return $attachments;
    }

    /**
     * @param $ver
     * @param string $operator
     *
     * @return mixed
     */
    public function versionCompare($ver, $operator = '>=')
    {
        $version = $this->getShopVersion();
        return version_compare($version, $ver, $operator);
    }

    /**
     *
     * @return array
     */
    public function getDefaultResponseData()
    {
        return [
            'meta_shopversion' => $this->getShopVersion(),
            'meta_modulversion' => $this->getModulVersion()
        ];
    }

    /**
     *
     * @return \Magento\Framework\DataObject
     */
    public function getDefaultResponseObject()
    {
        $dataObj = new DataObject();
        $dataObj->addData($this->getDefaultResponseData());
        return $dataObj;
    }
}
