<?php
/**
 *
 *  SenderBuilder.php
 *
 * @author   dlo
 * @copyright  CYBERDAY GmbH
 * @license    CYBERDAY GmbH
 * @version    0.1
 * @since    23.03.2016
 *
 *                          _(__)_        V
 *                        '-e e -'__,--.__)
 *                         (o_o)        )
 *                            \. /___.  |
 *                            ||| _)/_)/
 *                         dlo//_(/_(/_(
 *
 **/
namespace Cyberday\ApiItRechtKanzlei\Model\Rewrite\Order\Email;

use Psr\Log\LoggerInterface;
use Magento\Framework\Mail\Template\TransportBuilder;
use Magento\Sales\Model\Order\Email\Container\IdentityInterface;
use Magento\Sales\Model\Order\Email\Container\Template;
use Cyberday\ApiItRechtKanzlei\Model\Rewrite\Framework\Mail\Template\TransportBuilder as CyberdayTransportBuilder;
use Cyberday\ApiItRechtKanzlei\Helper\Data as Helper;
use \Throwable;
use \Exception;

class SenderBuilder extends \Magento\Sales\Model\Order\Email\SenderBuilder
{

    /**
     *
     * @var Psr\Log\LoggerInterface
     */
    protected $_logger;

    /**
     * @var CyberdayTransportBuilder
     */
    protected $transportBuilder;

    /**
     * Helper
     */
    protected $helper;

    /**
     *
     * @param Template $templateContainer
     * @param IdentityInterface $identityContainer
     * @param TransportBuilder $transportBuilder
     * @param LoggerInterface $logger
     */
    public function __construct(Template $templateContainer, IdentityInterface $identityContainer, TransportBuilder $transportBuilder, LoggerInterface $logger, Helper $helper)
    {
        parent::__construct($templateContainer, $identityContainer, $transportBuilder);
        $this->_logger = $logger;
        $this->helper = $helper;
    }

    /**
     *
     * {@inheritDoc}
     *
     * @see \Magento\Sales\Model\Order\Email\SenderBuilder::send()
     */
    public function send()
    {
        try {
            $templateVars = $this->templateContainer->getTemplateVars();
            if (isset($templateVars['attachments'])) {
                $attachments = $templateVars['attachments'];
                if (! is_array($attachments)) {
                    $attachments = [
                        $attachments
                    ];
                }

                foreach ($attachments as $attachment) {
                    if (! file_exists($attachment)) {
                        continue;
                    }

                    $content = @file_get_contents($attachment);
                    if($content === false || empty($content))  {
                        continue;
                    }

                    $name = basename($attachment);
                    
                    if($this->helper->versionCompare('2.2.8','<')) {
                        $this->transportBuilder->addAttachment($content, $name);
                    }elseif($this->helper->versionCompare('2.3.3','>=')) {
                        $this->transportBuilder->addBodyAttachment($content, $name, 'application/octet-stream');
                    } else {
                        $this->transportBuilder->getMessage()->setBodyAttachment($content, $name, 'application/octet-stream');
                    }
                }
            }
        } catch (Exception $exception) {
            $this->_logger->critical($exception);
        } catch (Throwable $exception) {
            $this->_logger->critical($exception);
        }
        parent::send();
    }
}
